import asyncio
import threading
from abc import ABC, abstractmethod
from concurrent.futures import ThreadPoolExecutor
from typing import List, Optional

from whiskerrag_types.model.multi_modal import Image


class BaseEmbedding(ABC):

    @classmethod
    def sync_health_check(cls) -> bool:
        def run_async_in_thread() -> bool:
            loop = asyncio.new_event_loop()
            asyncio.set_event_loop(loop)
            try:
                return loop.run_until_complete(cls.health_check())
            finally:
                loop.close()

        if threading.current_thread() is threading.main_thread():
            with ThreadPoolExecutor(max_workers=1) as executor:
                future = executor.submit(run_async_in_thread)
                return future.result()
        else:
            return run_async_in_thread()

    @classmethod
    @abstractmethod
    async def health_check(cls) -> bool:
        pass

    @abstractmethod
    async def embed_text(self, text: str, timeout: Optional[int]) -> List[float]:
        pass

    @abstractmethod
    async def embed_image(self, image: Image, timeout: Optional[int]) -> List[float]:
        pass



from typing import List

from whiskerrag_types.model.chunk import Chunk
from whiskerrag_types.model.knowledge import Knowledge
from whiskerrag_types.model.multi_modal import Image, Text

from .registry import RegisterTypeEnum, get_register, init_register, register


async def get_chunks_by_knowledge(knowledge: Knowledge) -> List[Chunk]:
    """
    Convert knowledge into vectorized chunks

    Args:
        knowledge (Knowledge): Knowledge object containing source type, split configuration,
                             embedding model and other information

    Returns:
        List[Chunk]: List of vectorized chunks

    Process flow:
    1. Get corresponding loader based on knowledge source type
    2. Get text splitter
    3. Get embedding model
    4. Load content
    5. Split content
    6. Vectorize each split content
    7. Generate final list of Chunk objects
    """
    LoaderCls = get_register(RegisterTypeEnum.KNOWLEDGE_LOADER, knowledge.source_type)
    split_type = getattr(knowledge.split_config, "type", None) or "base"
    SplitterCls = get_register(RegisterTypeEnum.SPLITTER, split_type)
    EmbeddingCls = get_register(
        RegisterTypeEnum.EMBEDDING, knowledge.embedding_model_name
    )
    contents = await LoaderCls(knowledge).load()
    parse_results = []
    for content in contents:
        split_result = SplitterCls().split(content, knowledge.split_config)
        parse_results.extend(split_result)
    chunks = []
    for parseItem in parse_results:
        if isinstance(parseItem, Text):
            embedding = await EmbeddingCls().embed_text(parseItem.content, timeout=30)
        elif isinstance(parseItem, Image):
            embedding = await EmbeddingCls().embed_image(parseItem, timeout=30)
        else:
            print(f"[warn]: illegal split item :{parseItem}")
            continue
        merged_metadata = {**knowledge.metadata, **(parseItem.metadata or {})}
        chunk = Chunk(
            context=parseItem.content,
            metadata=merged_metadata,
            embedding=embedding,
            knowledge_id=knowledge.knowledge_id,
            embedding_model_name=knowledge.embedding_model_name,
            space_id=knowledge.space_id,
            tenant_id=knowledge.tenant_id,
        )
        chunks.append(chunk)
    return chunks


__all__ = [
    "get_register",
    "register",
    "RegisterTypeEnum",
    "init_register",
    "SplitterEnum",
    "get_chunks_by_knowledge",
]



import hashlib
from datetime import datetime, timezone

from dateutil import parser


def parse_datetime(value: str) -> datetime:
    try:
        dt: datetime = parser.parse(value)
        if dt.tzinfo is None:
            dt = dt.replace(tzinfo=timezone.utc)
        return dt
    except Exception as e:
        raise ValueError(f"Invalid datetime format: {value}") from e


def calculate_sha256(text: str) -> str:
    text_bytes = text.encode("utf-8")
    sha256_hash = hashlib.sha256()
    sha256_hash.update(text_bytes)
    return sha256_hash.hexdigest()


__all__ = ["parse_datetime", "calculate_sha256"]



from abc import ABC, abstractmethod
from typing import Any


class LoggerManagerInterface(ABC):
    @abstractmethod
    def get_logger(self) -> Any:
        pass

    @abstractmethod
    def info(self, message: str, *args: tuple, **kwargs: dict) -> None:
        pass

    @abstractmethod
    def error(self, message: str, *args: tuple, **kwargs: dict) -> None:
        pass

    @abstractmethod
    def debug(self, message: str, *args: tuple, **kwargs: dict) -> None:
        pass

    @abstractmethod
    def warning(self, message: str, *args: tuple, **kwargs: dict) -> None:
        pass
